# Copyright (C) 2021 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# stratis.py - Boom Stratis storage integration
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
"""The ``boom.stratis`` module contains functions and constants
needed to obtain information from a running ``stratisd`` daemon
about the Stratis storage pools and file systems present on the
system.

This includes the ability to obtain the stratis ``pool_uuid``
value for a given Stratis storage pool name. This is needed to
set the ``stratis.rootfs.pool_uuid`` kernel command line argument
that is read by the Stratis early userspace systemd generator.
"""
from __future__ import print_function

import logging
from os import sep as path_sep
from os.path import normpath
from uuid import UUID
import dbus

from boom import *

# Module logging configuration
_log = logging.getLogger(__name__)
_log.set_debug_mask(BOOM_DEBUG_STRATIS)

_log_debug = _log.debug
_log_debug_stratis = _log.debug_masked
_log_info = _log.info
_log_warn = _log.warning
_log_error = _log.error

# Constants for the Stratisd DBus service

#: The DBus name of the stratisd service
_STRATISD_SERVICE = "org.storage.stratis2"
#: The path to the stratisd service object
_STRATISD_PATH = "/org/storage/stratis2"
#: The DBus name of the pool interface
_POOL_IFACE = "org.storage.stratis2.pool"
#: The DBus timeout for stratisd in miliseconds
_STRATISD_TIMEOUT = 120000

#: The DBus ObjectManager interface implemented by stratisd
_DBUS_OBJECT_MANAGER_IFACE = "org.freedesktop.DBus.ObjectManager"


def pool_name_to_pool_uuid(pool_name):
    """Return the UUID of the pool named ``pool_name`` as a string.

        :param pool_name: The name of the Stratis pool.
        :returns: A string representation of the pool UUID value. The
                  returned string contains the un-formatted character
                  sequence that makes up the pool UUID.

        :rtype: str
    """
    bus = dbus.SystemBus()
    _log_debug_stratis("Connecting to %s at %s via system bus" %
                       (_STRATISD_SERVICE, _STRATISD_PATH))
    proxy = bus.get_object(_STRATISD_SERVICE, _STRATISD_PATH,
                           introspect=False)
    object_manager = dbus.Interface(proxy, _DBUS_OBJECT_MANAGER_IFACE)
    managed_objects = object_manager.GetManagedObjects(_STRATISD_TIMEOUT)
    try:
        props = next(
            obj_data[_POOL_IFACE]
            for _, obj_data in managed_objects.items()
            if _POOL_IFACE in obj_data and
            obj_data[_POOL_IFACE]["Name"] == pool_name
        )
    except StopIteration as e:
        raise IndexError("Stratis pool '%s' not found" % pool_name)
    pool_uuid = str(props["Uuid"])
    _log_debug("Looked up pool_uuid=%s for Stratis pool %s" %
               (format_pool_uuid(pool_uuid), pool_name))
    return pool_uuid


def symlink_to_pool_uuid(link_path):
    """Return the UUID of the pool corresponding to the stratis file system
       at ``link_path`` as a string.

       :param link_path: A symbolic link corresponding to a Stratis file
                        system on the local system.
       :returns: A string representation of the UUID value. The returned
                 string contains the un-formatted character sequence that
                 makes up the pool UUID.
       :rtype: str
    """
    # Separate the "pool" and "fs" components from a Stratis file system
    # link path formatted as "/dev/stratis/pool/ps".
    (pool, fs) = normpath(link_path).split(path_sep)[-2:]
    _log_debug_stratis("Looking up pool UUID for Stratis symlink '%s'" %
                       link_path)
    return pool_name_to_pool_uuid(pool)


def format_pool_uuid(pool_uuid):
    """Return the UUID ``pool_uuid`` formatted as a hyphen-separated
        string.

        :param pool_uuid: The UUID value to format.
        :returns: A hyphen-separated string representation of the UUID.
        :rtype: str
    """
    uuid = UUID(pool_uuid)
    return str(uuid)


def is_stratis_device_path(dev_path):
    prefix = "/dev/stratis/"
    if not normpath(dev_path).startswith(prefix):
        return False
    try:
        pool_uuid = symlink_to_pool_uuid(dev_path)
    except (dbus.DBusException, IndexError):
        return False
    return True


__all__ = [
    "symlink_to_pool_uuid", "pool_name_to_pool_uuid",
    "format_pool_uuid", "is_stratis_device_path"
]

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# config_tests.py - Boom report API tests.
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
import unittest
import logging
from os.path import abspath, join
from sys import stdout
import shutil

try:
    # Python2
    from ConfigParser import SafeConfigParser as ConfigParser, ParsingError
except:
    # Python3
    from configparser import ConfigParser, ParsingError

log = logging.getLogger()
log.level = logging.DEBUG
log.addHandler(logging.FileHandler("test.log"))

# Test suite paths
from tests import *

from boom import *
from boom.config import *
BOOT_ROOT_TEST = abspath("./tests")
set_boot_path(BOOT_ROOT_TEST)

class ConfigBasicTests(unittest.TestCase):
    """Basic tests for the boom.config sub-module.
    """

    def test_sync_config(self):
        """Test that the internal _sync_config() helper works.
        """
        import boom.config # for _sync_config()
        cfg = ConfigParser()
        bc = BoomConfig()

        cfg.add_section("global")
        cfg.add_section("legacy")

        boot_path = "/boot"
        boom_path = "/boot/boom"
        legacy_format = "grub1"

        bc.legacy_enabled = False
        bc.legacy_sync = False
        bc.legacy_format = legacy_format

        bc.boot_path = boot_path
        bc.boom_path = boom_path

        boom.config._sync_config(bc, cfg)
        self.assertEqual(cfg.get("legacy", "enable"), "no")
        self.assertEqual(cfg.get("legacy", "sync"), "no")
        self.assertEqual(cfg.get("legacy", "format"), legacy_format)
        self.assertEqual(cfg.get("global", "boot_root"), boot_path)
        self.assertEqual(cfg.get("global", "boom_root"), boom_path)


class ConfigTests(unittest.TestCase):
    # The set of configuration files to use for this test class
    conf_path = join(BOOT_ROOT_TEST, "boom_configs/default/boot")

    # The path to the boot directory in the test sandbox
    boot_path = join(SANDBOX_PATH, "boot")

    # The path to the sandbox boom.conf configuration file
    boom_conf = join(boot_path, "boom/boom.conf")

    def setUp(self):
        """Set up a test fixture for the ConfigTests class.
        """
        reset_sandbox()

        # Sandbox paths
        shutil.copytree(self.conf_path, join(SANDBOX_PATH, "boot"))
        # Set boom paths
        set_boot_path(self.boot_path)

    def tearDown(self):
        rm_sandbox()
        reset_boom_paths()

    def test_get_boom_config_path(self):
        """Test that the correct boom.conf path is returned from a call
            to the `get_boom_config_path()` function.
        """
        conf_path = self.boom_conf
        self.assertEqual(get_boom_config_path(), conf_path)

    def test_set_boom_config_path_abs(self):
        """Test that the correct boom.conf path is returned from a call
            to the `get_boom_config_path()` function when an absolute
            path is given.
        """
        conf_dir = join(SANDBOX_PATH, "boot/boom")
        conf_path = join(conf_dir, "boom.conf")
        set_boom_config_path(conf_dir)
        self.assertEqual(get_boom_config_path(), conf_path)

    def test_load_boom_config_default(self):
        """Test the `load_boom_config()` function with the default
            configuration file.
        """
        load_boom_config()

class BadConfigTests(ConfigTests):
    # The set of configuration files to use for this test class
    conf_path = join(BOOT_ROOT_TEST, "boom_configs/badconfig/boot")

    def test_load_boom_config_default(self):
        """Test the `load_boom_config()` function with the default
            configuration file.
        """
        with self.assertRaises(ValueError) as cm:
            load_boom_config()

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# osprofile_tests.py - Boom OS profile tests.
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
import unittest
import logging
from sys import stdout
from os import listdir, makedirs, mknod, unlink
from os.path import abspath, exists, join
from stat import S_IFBLK, S_IFCHR
import shutil

# Test suite paths
from tests import *

import boom
from boom.stratis import *

# Override default BOOM_ROOT and BOOT_ROOT
# NOTE: with test fixtures that use the sandbox, this path is further
# overridden by the class setUp() method to point to the appropriate
# sandbox location.
boom.set_boot_path(BOOT_ROOT_TEST)

log = logging.getLogger()
log.level = logging.DEBUG
log.addHandler(logging.FileHandler("test.log"))


class StratisTests(unittest.TestCase):
    """Tests for the BootEntry class that do not depend on external
        test data.
    """
    # Stratis module tests

    def test_is_stratis_device_path_badpath(self):
        self.assertEqual(is_stratis_device_path("/dev/notstratis/foo"), False)

    def test_is_stratis_device_path_nosuchpool(self):
        self.assertEqual(is_stratis_device_path("/dev/stratis/nosuchpool/fs1"), False)

    def test_format_pool_uuid_valid(self):
        uuid_val = "b4580e1e30b0424e8efc7e578698fa8f"
        uuid_xval = "b4580e1e-30b0-424e-8efc-7e578698fa8f"
        self.assertEqual(format_pool_uuid(uuid_val), uuid_xval)

    def test_format_pool_uuid_baduuid(self):
        uuid_val = "QUUX"
        with self.assertRaises(ValueError) as cm:
            format_pool_uuid(uuid_val)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# cache.py - Boom boot image cache
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
"""The ``boom.cache`` module defines classes, constants and functions
for maintaining an on-disk cache of kernel, initramfs and auxiliary
images required to load boom-defined boot entries.
"""
from __future__ import print_function

from boom import *
from boom.bootloader import *

from hashlib import sha1
from os import chmod, chown, fdatasync, listdir, stat, unlink
from stat import S_ISREG, ST_MODE, ST_UID, ST_GID, ST_MTIME, filemode
from os.path import (
    join as path_join, exists as path_exists, sep as path_sep,
    basename, dirname
)
from json import load as json_load, dump as json_dump
from errno import ENOENT
import shutil
import logging

# Module logging configuration
_log = logging.getLogger(__name__)
_log.set_debug_mask(BOOM_DEBUG_CACHE)

_log_debug = _log.debug
_log_debug_cache = _log.debug_masked
_log_info = _log.info
_log_warn = _log.warning
_log_error = _log.error

#: Block size for hashing image files
_hash_size = 1024**2

#: The name of the JSON cache index file
_CACHE_INDEX = "cacheindex.json"

#: The extension used for cached image files
_IMAGE_EXT = ".img"

#
# Constants for access to cache metadata dictionaries
#

#: Path timestamp
PATH_TS = "path_ts"
#: Path mode
PATH_MODE = "path_mode"
#: Path user ID
PATH_UID = "path_uid"
#: Path group ID
PATH_GID = "path_gid"
#: Path attribute map
PATH_ATTRS = "path_attrs"

#: Image timestamp
IMAGE_TS = "image_ts"

# Cache states
#: Path is cached
CACHE_CACHED = "CACHED"
#: Path is cached and missing from /boot
CACHE_MISSING = "MISSING"
#: Path is cached but image missing or damaged
CACHE_BROKEN = "BROKEN"
#: Path is cached and has been restored to /boot
CACHE_RESTORED = "RESTORED"

#
# Path names in the boom boot image cache
#
# img_path  -  The path to an image as specified in a BLS snippet, or an
#              argument to cache_path(), relative to the root of the
#              /boot file system. E.g. "/vmlinuz-5.0.0".
#
# boot_path  - The absolute path to an image relative to the host root
#              file system. E.g. "/boot/vmlinuz-5.0.0".
#
# cache_path - The absolute path to a cached boot image relative to the
#              host root file system. E.g.
#              "/boot/boom/cache/1562375e4d022e814ba39521d0852e490b7c07f8.img"
#

#: The index of img_path names to lists of image identifiers.
_index = {}

#: Mapping of img_path to path metadata dictionaries
_paths = {}

#: Mapping of image identifiers to image metadata.
_images = {}


def _make_relative(img_path):
    """Convert an image path with a leading path separator into
        a relative path fragment suitable for passing to the
        os.path.join() function.

        :param img_path: The path to convert.
        :returns: The path without any leading path separator.
    """
    if img_path[0] == path_sep:
        img_path = img_path[1:]
    return img_path


def _image_path_to_boot(img_path):
    """Convert an image path relative to /boot into an absolute
        path to the corresponding boot image.

        :param img_path: The path to the image relative to the
                         /boot file system.
        :returns: The absolute path to the image including the
                  current /boot file system mount point.
        :rtype: str
    """
    img_path = _make_relative(img_path)
    return path_join(get_boot_path(), img_path)


def _image_id_to_cache_path(img_id):
    """Convert an image path relative to /boot into a path
        for the corresponding cache entry.

        :param img_id: The SHA1 digest of the image
        :returns: The cache path for the image found at ``img_path``.
        :rtype: str
    """
    return path_join(get_cache_path(), "%s%s" % (img_id, _IMAGE_EXT))


def _image_id_from_file(img_file):
    """Calculate the image identifier (SHA1) for the image file
        open at ``img_file``.

        :param img_file: An file-like object open for binary reading.
        :returns: The image identifier for ``img_file``.
        :rtype: str
    """
    digest = sha1()
    while True:
        hashdata = img_file.read(_hash_size)
        if not hashdata:
            break
        digest.update(hashdata)
    return digest.hexdigest()


def _image_id_from_path(img_path):
    """Calculate the image identifier (SHA1) for the image found
        at ``img_path``.

        :param img_path: The absolute path to the on-disk image.
        :returns: The image identifier for ``img_path``.
        :rtype: str
    """
    with open(img_path, "rb") as img_file:
        return _image_id_from_file(img_file)


def drop_cache():
    """Discard the in-memory cache state. Calling this function has
        no effect on the persistent cache state but will free all
        in-memory represenatations and clear the cache index.
    """
    global _index, _paths, _images
    _index = {}
    _paths = {}
    _images = {}


def _load_image_ids(cache_path):
    """Read the set of image_id values from the cache directory.

        :returns: A list of image_id values.
    """
    ids = []
    for entry in listdir(cache_path):
        if not entry.endswith(_IMAGE_EXT):
            continue
        ids.append(entry.rstrip(_IMAGE_EXT))
    return ids


def load_cache(verify=True, digests=False):
    """Read the state of the on-disk boot image cache into memory.
    """
    global _index, _paths, _images
    drop_cache()

    cache_path = get_cache_path()

    index_path = path_join(cache_path, _CACHE_INDEX)

    _log_debug("Loading cache entries from '%s'" % index_path)

    # Get the set of known image_id values
    ids = _load_image_ids(cache_path)

    cachedata = {}
    try:
        with open(index_path, "r") as index_file:
            cachedata = json_load(index_file)
    except IOError as e:
        if e.errno != ENOENT:
            raise e
        _log_debug("No metadata found: starting empty cache")
        return

    index = cachedata["index"]

    for path in index.keys():
        for image_id in index[path]:
            if image_id not in ids:
                _log_warn("Image identifier '%s' not found in cache"
                          " for path %s" % (image_id, path))
                # mark as broken

    paths = cachedata["paths"]
    for path in paths.keys():
        if path not in index:
            _log_warn("No image for path '%s' found in cache" % path)
            # mark as broken

    images = cachedata["images"]
    for image_id in images.keys():
        if image_id not in ids:
            _log_warn("Found unknown image_id '%s'" % image_id)
            # clean up?

    _log_debug("Loaded %d cache paths and %d images" %
               (len(paths), len(sum(index.values(), []))))

    _index = index
    _paths = paths
    _images = images


def write_cache():
    """Write the current in-memory state of the cache to disk.
    """
    cache_path = get_cache_path()

    index_path = path_join(cache_path, _CACHE_INDEX)

    cachedata = {
        "index": _index,
        "paths": _paths,
        "images": _images
    }

    with open(index_path, "w") as index_file:
        json_dump(cachedata, index_file)
        fdatasync(index_file.fileno())


def _insert_copy(boot_path, cache_path):
    """Insert an image into the cache by physical data copy.
    """
    shutil.copy2(boot_path, cache_path)


def _insert(boot_path, cache_path):
    """Insert an image into the cache.

        :param boot_path: The absolute path to the image to add.
        :param cache_path: The cache path at which to insert.
        :returns: None
    """
    try:
        # FIXME: implement hard link support with fall-back to copy.
        _insert_copy(boot_path, cache_path)
    except Exception as e:
        _log_error("Error copying '%s' to cache: %s" % (boot_path, e))
        raise e


def _remove_boot(boot_path):
    """Remove a boom restored boot image from /boot.
    """
    boot_dir = dirname(boot_path)
    dot_path = _RESTORED_DOT_PATTERN % basename(boot_path)
    if not path_exists(path_join(boot_dir, dot_path)):
        raise ValueError("'%s' is not boom managed" % boot_path)
    unlink(boot_path)
    unlink(path_join(boot_dir, dot_path))


def _remove_copy(cache_path):
    """Remove an image copy from the cache store.
    """
    unlink(cache_path)


def _remove(cache_path):
    """Remove an image from the cache store.

        :param cache_path: The path to the image to be removed.
        :returns: None
    """
    if not cache_path.startswith(get_cache_path()):
        raise ValueError("'%s' is not a boom cache path" % cache_path)
    try:
        _remove_copy(cache_path)
    except Exception as e:
        _log_error("Error removing cache image '%s': %s" %
                   (cache_path, e))
        raise e


def _insert_path(path, img_id, mode, uid, gid, attrs):
    """Insert a path into the path map and index dictionaries.
    """
    global _paths, _index

    if path not in _paths:
        _paths[path] = {}
    _paths[path][PATH_MODE] = mode
    _paths[path][PATH_UID] = uid
    _paths[path][PATH_GID] = gid
    _paths[path][PATH_ATTRS] = attrs

    # Add the img_id to the list of images for this path
    if path in _index and img_id not in _index[path]:
        _index[path].append(img_id)
    else:
        _index[path] = [img_id]


def _cache_path(img_path, update=True, backup=None):
    """Add an image to the boom boot image cache.

        :param img_path: The path to the on-disk boot image relative to
                         the configured /boot directory.
        :returns: None
    """
    def this_entry():
        """Return a new CacheEntry object representing the newly cached
            path.
        """
        return CacheEntry(img_path, _paths[img_path], [(img_id, image_ts)])

    global _index, _paths, _images

    boot_path = _image_path_to_boot(img_path)
    st = stat(boot_path)

    if not S_ISREG(st[ST_MODE]):
        _log_error("Image at path '%s' is not a regular file." % img_path)
        raise ValueError("'%s' is not a regular file" % img_path)

    img_id = _image_id_from_path(boot_path)
    cache_path = _image_id_to_cache_path(img_id)
    image_ts = st[ST_MTIME]

    img_path = backup or img_path

    # Already present?
    if img_path in _paths:
        _log_info("Path '%s' already cached." % img_path)
        if not update:
            return

    if img_path in _paths and img_id in _index[img_path]:
        _log_info("Image with img_id=%s already cached for path '%s'" %
                  (img_id[0:6], img_path))
        return this_entry()
    _log_info("Adding new image with img_id=%s for path '%s'" %
              (img_id[0:6], img_path))

    path_mode = st[ST_MODE]
    path_uid = st[ST_UID]
    path_gid = st[ST_GID]
    path_attrs = {}  # FIXME xattr support

    # Physically cache the image
    _insert_copy(boot_path, cache_path)

    # Set cache entry metadata
    _images[img_id] = {IMAGE_TS: image_ts}
    _insert_path(img_path, img_id, path_mode, path_uid, path_gid, path_attrs)
    write_cache()

    return this_entry()


def cache_path(img_path, update=True):
    """Add an image to the boom boot image cache.

        :param img_path: The path to the on-disk boot image relative to
                         the configured /boot directory.
        :returns: None
    """
    _log_debug_cache("Caching path '%s'" % img_path)
    return _cache_path(img_path, update=update)


def backup_path(img_path, backup_path):
    """Back up an image to the boom boot image cache.

        :param img_path: The path to the on-disk boot image relative to
                         the configured /boot directory.
        :param backup_path: The path where the backup image will be created.
        :returns: None
    """
    _log_debug_cache("Backing up path '%s' as '%s'" % (img_path, backup_path))
    ce = _cache_path(img_path, backup=backup_path)
    ce.restore()
    return ce


def uncache_path(img_path, force=False):
    """Remove paths from the boot image cache.

        Remove ``img_path`` from the boot image cache and discard any
        unused images referenced by the cache entry. Images that are
        shared with other cached paths are not removed.

        :param img_path: The cached path to remove
    """
    global _index, _paths, _images

    if img_path not in _paths:
        raise ValueError("Path '%s' is not cached." % img_path)

    boot_path = _image_path_to_boot(img_path)
    img_id = _index[img_path][0]
    ts = _images[img_id][IMAGE_TS]

    ce = CacheEntry(img_path, _paths[img_path], [(img_id, ts)])
    count = ce.count

    if count and not force:
        _log_info("Retaining cache path '%s' used by %d boot entries" %
                  (img_path, count))
        return

    if count:
        _log_warn("Uncaching path '%s' used by %d boot entries"
                  % (img_path, count))

    # Remove entry from the path index and metadata
    images = _index.pop(img_path)
    _paths.pop(img_path)
    # Clean up unused images
    for img_id in images:
        all_images = sum(_index.values(), [])
        # Shared image?
        if img_id not in all_images:
            _images.pop(img_id)
            cache_path = _image_id_to_cache_path(img_id)
            _remove(cache_path)
    if _is_restored(boot_path):
        _remove_boot(boot_path)

    write_cache()


def clean_cache():
    """Remove unused cache entries.

        Iterate over the set of cache entries and remove any paths
        that are not referenced by any BootEntry, and remove all
        images that are not referenced by a path.
    """
    ces = find_cache_paths()
    nr_unused = 0
    for ce in ces:
        if not ce.count:
            nr_unused += 1
            ce.uncache()
    if nr_unused:
        _log_info("Removed %d unused cache entries" % nr_unused)


#: Boom restored dot file pattern
_RESTORED_DOT_PATTERN = ".%s.boomrestored"


def _is_restored(boot_path):
    """Return ``True`` if ``boot_path`` was restored by boom, or
        ``False`` otherwise.

        :param boot_path: The absolute path to a boot image.
    """
    boot_dir = dirname(boot_path)
    dot_path = _RESTORED_DOT_PATTERN % basename(boot_path)
    return path_exists(path_join(boot_dir, dot_path))


class CacheEntry(object):
    """In-memory representation of cached boot image.
    """
    #: The image path for this CacheEntry
    path = None

    @property
    def img_id(self):
        return self.images[0][0]

    @property
    def disp_img_id(self):
        shas = set([img_id for (img_id, ts) in self.images])
        width = find_minimum_sha_prefix(shas, 7)
        return self.images[0][0][0:width]

    @property
    def mode(self):
        """The file system mode for this CacheEntry.
        """
        return self._pathdata[PATH_MODE]

    @property
    def uid(self):
        """The file system uid for this CacheEntry.
        """
        return self._pathdata[PATH_UID]

    @property
    def gid(self):
        """The file system gid for this CacheEntry.
        """
        return self._pathdata[PATH_GID]

    @property
    def attrs(self):
        """The dictionary of extended attrs for this CacheEntry.
        """
        return self._pathdata[PATH_ATTRS]

    @property
    def timestamp(self):
        """The timestamp of the most recent image for this CacheEntry.
        """
        return self.images[0][1]

    @property
    def state(self):
        """Return a string representing the state of this cache entry.
        """
        boot_path = _image_path_to_boot(self.path)
        cache_path = _image_id_to_cache_path(self.images[0][0])
        boot_exists = path_exists(boot_path)
        cache_exists = path_exists(cache_path)
        if boot_exists and cache_exists:
            boot_path_id = _image_id_from_path(boot_path)
            if _is_restored(boot_path) and self.img_id == boot_path_id:
                return CACHE_RESTORED
            else:
                return CACHE_CACHED
        if cache_exists and not boot_exists:
            return CACHE_MISSING
        if boot_exists and not cache_exists:
            return CACHE_BROKEN
        return CACHE_UNKNOWN

    @property
    def count(self):
        """Return the current number of boot entries that reference
            this cache entry.
        """
        entries = find_entries(Selection(path=self.path))
        # Ignore foreign boot entries
        entries = [entry for entry in entries if not entry.read_only]
        return len(entries)

    #: The list of cached images for this CacheEntry sorted by increasing age
    images = []

    def __init__(self, path, pathdata, images):
        """Initialise a CacheEntry object with information from
            the on-disk cache.
        """
        self.path = path
        self._pathdata = pathdata
        self.images = images

    def __str__(self):
        fmt = "Path: %s\nImage ID: %s\nMode: %s\nUid: %d Gid: %d\nTs: %d"
        return fmt % (
            self.path,
            self.disp_img_id,
            filemode(self.mode),
            self.uid, self.gid,
            self.timestamp
        )

    def __repr__(self):
        shas = set([img_id for (img_id, ts) in self.images])
        width = find_minimum_sha_prefix(shas, 7)
        rep = '"%s", %s, %s' % (
            self.path,
            self._pathdata,
            # FIXME: properly generate minimum abrevs
            [(img_id[0:width - 1], ts) for (img_id, ts) in self.images]
        )
        return "CacheEntry(" + rep + ")"

    def restore(self, dest=None):
        """Restore this CacheEntry to the /boot file system.
        """
        img_id = self.images[0][0]
        if dest:
            if dest not in _index:
                _insert_path(dest, img_id, self.mode, self.uid, self.gid,
                             self.attrs)
            self.path = dest
            write_cache()
        boot_path = _image_path_to_boot(self.path)
        cache_path = _image_id_to_cache_path(img_id)
        dot_path = _RESTORED_DOT_PATTERN % basename(boot_path)
        boot_dir = dirname(boot_path)

        restore_states = (CACHE_MISSING, CACHE_RESTORED)
        if self.state not in restore_states:
            raise ValueError("Restore failed: CacheEntry state is not "
                             "%s or %s" % restore_states)

        shutil.copy2(cache_path, boot_path)
        try:
            chown(boot_path, self.uid, self.gid)
            chmod(boot_path, self.mode)
        except OSError as e:
            try:
                unlink(boot_path)
            except OSError:
                pass
            raise e

        try:
            dot_file = open(path_join(boot_dir, dot_path), "w")
            dot_file.close()
        except OSError as e:
            try:
                unlink(boot_path)
            except OSError:
                pass
            raise e

    def purge(self):
        """Remove the boom restored image copy from the /boot file system.
        """
        boot_path = _image_path_to_boot(self.path)
        if self.state is not CACHE_RESTORED:
            raise ValueError("Purge failed: CacheEntry state is not RESTORED")
        _remove_boot(boot_path)

    def uncache(self):
        """Remove this CacheEntry from the boot image cache.
        """
        uncache_path(self.path)


def select_cache_entry(s, ce):
    """Test CacheEntry against Selection criteria.

        Test the supplied ``CacheEntry`` against the selection criteria
        in ``s`` and return ``True`` if it passes, or ``False``
        otherwise.

        :param s: The selection criteria
        :param be: The CacheEntry to test
        :rtype: bool
        :returns: True if CacheEntry passes selection or ``False``
                  otherwise.
    """
    # Version matches if version string is contained in image path.
    if s.version and s.version not in ce.path:
        return False

    # Image path match is an exact match.
    if s.linux and s.linux != ce.path:
        return False
    if s.initrd and s.initrd != ce.path:
        return False
    if s.path and s.path != ce.path:
        return False
    if s.timestamp and s.timestamp != ce.timestamp:
        return False
    if s.img_id and s.img_id not in ce.img_id:
        return False
    return True


def _find_cache_entries(selection=None, images=False):
    """Find cache entries matching selection criteria.

        Return a list of ``CacheEntry`` objects matching the supplied
        ``selection`` criteria. If ``images`` is ``True`` a separate
        entry is returned for each image in the cache: otherwise one
        ``CacheEntry`` object is returned for each cached path.

        ;param selection: cache entry selection criteria.
        :param images: return results by images instead of paths.
        :returns: A list of matching ``CacheEntry`` objects.
    """
    global _index, _paths, _images
    matches = []

    if not _index:
        load_cache()

    # Use null search criteria if unspecified
    selection = selection if selection else Selection()

    selection.check_valid_selection(cache=True)

    _log_debug_cache("Finding cache entries for %s" % repr(selection))

    for path in _index:
        def ts_key(val):
            return val[1]

        def tuplicate(img_id):
            """Return a (img_id, image_ts) tuple for the given img_id.
            """
            return (img_id, _images[img_id][IMAGE_TS])

        entry_images = [tuplicate(img_id) for img_id in _index[path]]
        # Sort images list from newest to oldest
        entry_images = sorted(entry_images, reverse=True, key=ts_key)
        if not images:
            ce = CacheEntry(path, _paths[path], entry_images)
            if select_cache_entry(selection, ce):
                matches.append(ce)
        else:
            for img_tuple in entry_images:
                ce = CacheEntry(path, _paths[path], [img_tuple])
                if select_cache_entry(selection, ce):
                    matches.append(ce)

    return matches


def find_cache_paths(selection=None):
    """Find cache entries matching selection criteria.

        Return a list of ``CacheEntry`` objects matching the supplied
        ``selection`` criteria, one for each path that exists in the
        cache. For each cached path a ``CacheEntry`` object is returned
        with a list of images that are cached for that path. The image
        list is sorted by timestamp with the most recent entry first.

        ;param selection: cache entry selection criteria.
        :returns: A list of matching ``CacheEntry`` objects.
    """
    matches = _find_cache_entries(selection=selection, images=False)
    _log_debug_cache("Found %d cached paths" % len(matches))
    return matches


def find_cache_images(selection=None):
    """Find cache entries matching selection criteria.

        Return a list of ``CacheEntry`` objects matching the supplied
        ``selection`` criteria, one for each image that exists in the
        cache. Each ``CacheEntry`` object is returned with a list
        containing a single image.

        ;param selection: cache entry selection criteria.
        :returns: A list of matching ``CacheEntry`` objects.
    """
    matches = _find_cache_entries(selection=selection, images=True)
    _log_debug_cache("Found %d cached images" % len(matches))
    return matches


__all__ = [
    "CACHE_CACHED", "CACHE_MISSING", "CACHE_BROKEN", "CACHE_RESTORED",
    "drop_cache", "load_cache", "write_cache",
    "cache_path", "backup_path", "uncache_path", "clean_cache",
    "find_cache_paths", "find_cache_images"
]

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# cache_tests.py - Boom cache API tests.
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
import unittest
import logging
from sys import stdout
from os import listdir, makedirs, unlink
from os.path import abspath, basename, dirname, exists, join
from glob import glob
import shutil
import re

# Python3 moves StringIO to io
try:
    from StringIO import StringIO
except:
    from io import StringIO

log = logging.getLogger()
log.level = logging.DEBUG
log.addHandler(logging.FileHandler("test.log"))

from boom import *
from boom.osprofile import *
from boom.bootloader import *
from boom.hostprofile import *
from boom.command import *
from boom.config import *
from boom.report import *
from boom.cache import *

# For access to non-exported members
import boom.cache

from tests import *

BOOT_ROOT_TEST = abspath("./tests")
config = BoomConfig()
config.legacy_enable = False
config.legacy_sync = False
set_boom_config(config)
set_boot_path(BOOT_ROOT_TEST)

debug_masks = ['profile', 'entry', 'report', 'command', 'all']


class CacheHelperTests(unittest.TestCase):
    """Test internal boom.cache helpers. Cases in this class must
        not modify on-disk state and do not use a unique test
        fixture.
    """

    # Test fixture init/cleanup
    def setUp(self):
        """Set up a test fixture for the CacheHelperTests class.
        """
        set_boom_config(config)
        set_boot_path(BOOT_ROOT_TEST)

    def tearDown(self):
        # Drop any in-memory entries and profiles modified by tests
        drop_entries()
        drop_profiles()
        drop_host_profiles()
        drop_cache()

    def test__make_relative_with_non_abs_path(self):
        path = "not/an/absolute/path"
        self.assertEqual(path, boom.cache._make_relative(path))

    def test__make_relative_with_abs_path(self):
        path = "/vmlinuz"
        self.assertEqual(path[1:], boom.cache._make_relative(path))

    def test__make_relative_root_is_empty_string(self):
        path = "/"
        self.assertEqual("", boom.cache._make_relative(path))

    def test__image_path_to_boot(self):
        image_path = "vmlinuz"
        xboot_path = join(get_boot_path(), image_path)
        boot_path = boom.cache._image_path_to_boot(join("/", image_path))
        self.assertEqual(boot_path, xboot_path)

    def test__image_id_to_cache_path(self):
        img_id = "47dc6ad4ea9ca5453e607987d49c33858bd553e0"
        xcache_file = "47dc6ad4ea9ca5453e607987d49c33858bd553e0.img"
        self.assertEqual(boom.cache._image_id_to_cache_path(img_id),
                         join(get_cache_path(), xcache_file))

    def test__image_id_from_path(self):
        img_path = join(get_boot_path(), "vmlinuz-5.5.5-1.fc30.x86_64")
        ximg_id = "fdfb8e5a3857adca47f25ee47078bad4a757cc92"
        self.assertEqual(boom.cache._image_id_from_path(img_path), ximg_id)

    def test__image_id_from_bad_path_raises(self):
        img_path = "/qux/qux/qux"  # non-existent
        with self.assertRaises(IOError) as cm:
            boom.cache._image_id_from_path(img_path)

class CacheTests(unittest.TestCase):
    """Test boom.command APIs
    """

    # Main BLS loader directory for sandbox
    loader_path = join(BOOT_ROOT_TEST, "loader")

    # Main boom configuration path for sandbox
    boom_path = join(BOOT_ROOT_TEST, "boom")

    # Main grub configuration path for sandbox
    grub_path = join(BOOT_ROOT_TEST, "grub")

    # Test fixture init/cleanup
    def setUp(self):
        """Set up a test fixture for the CommandTests class.

            Defines standard objects for use in these tests.
        """
        reset_sandbox()

        # Sandbox paths
        boot_sandbox = join(SANDBOX_PATH, "boot")
        boom_sandbox = join(SANDBOX_PATH, "boot/boom")
        grub_sandbox = join(SANDBOX_PATH, "boot/grub")
        loader_sandbox = join(SANDBOX_PATH, "boot/loader")

        # Initialise sandbox from master
        makedirs(boot_sandbox)
        shutil.copytree(self.boom_path, boom_sandbox)
        shutil.copytree(self.loader_path, loader_sandbox)
        shutil.copytree(self.grub_path, grub_sandbox)

        # Copy boot images
        images = glob(join(BOOT_ROOT_TEST, "initramfs*"))
        images += glob(join(BOOT_ROOT_TEST, "vmlinuz*"))
        for image in images:
            def _dotfile(img_path):
                pattern = ".%s.boomrestored"
                img_name = basename(img_path)
                img_dir = dirname(img_path)
                return join(img_dir, pattern % img_name)

            shutil.copy2(image, boot_sandbox)
            if exists(_dotfile(image)):
                shutil.copy2(_dotfile(image), boot_sandbox)

        # Set boom paths
        set_boot_path(boot_sandbox)

        # Tests that deal with legacy configs will enable this.
        config = BoomConfig()
        config.legacy_enable = False
        config.legacy_sync = False

        # Reset profiles, entries, and host profiles to known state.
        load_profiles()
        load_entries()
        load_host_profiles()
        load_cache()

    def tearDown(self):
        # Drop any in-memory entries and profiles modified by tests
        drop_entries()
        drop_profiles()
        drop_host_profiles()
        drop_cache()

        # Clear sandbox data
        rm_sandbox()
        reset_boom_paths()

    def _make_null_testimg(self, restored=False):
        """Return an empty file in the configured $BOOT path for test use.
            If ``restored`` is ``True`` the image will be made to look like
            an image restored by boom.
        """
        img_name = "testimg"
        boot_path = get_boot_path()
        print(boot_path)
        # The img_id of an empty file is the sha1 null hash: da39a3e
        img_file = open(join(boot_path, img_name), "w")
        img_file.close()

        if restored:
            boomrestored_name = "." + img_name + ".boomrestored"
            boomrestored_file = open(join(boot_path, boomrestored_name), "w")
            boomrestored_file.close()

        return img_name

    def test_drop_cache(self):
        drop_cache()
        self.assertEqual(len(boom.cache._index), 0)
        self.assertEqual(len(boom.cache._paths), 0)
        self.assertEqual(len(boom.cache._images), 0)

    def test_load_cache(self):
        load_cache()
        self.assertTrue(len(boom.cache._index))
        self.assertTrue(len(boom.cache._paths))
        self.assertTrue(len(boom.cache._images))

        # Verify number of images
        boom_cache_path = get_cache_path()
        ximage_count = 0
        for p in listdir(boom_cache_path):
            if p.endswith(".img"):
                ximage_count += 1
        self.assertEqual(len(boom.cache._images), ximage_count)

    def test_load_cache_no_cacheindex(self):
        # Wipe cache
        unlink(join(SANDBOX_PATH, "boot/boom/cache/cacheindex.json"))
        load_cache()
        self.assertFalse(len(boom.cache._index))
        self.assertFalse(len(boom.cache._paths))
        self.assertFalse(len(boom.cache._images))

    def test_write_cache(self):
        # Re-write the current cache state
        write_cache()

        # Write an empty cache
        drop_cache()
        write_cache()

    def test__insert(self):
        img_name = self._make_null_testimg()
        img_path = join(get_boot_path(), img_name)

        cache_name = "da39a3ee5e6b4b0d3255bfef95601890afd80709.img"
        cache_path = join(get_cache_path(), cache_name)

        boom.cache._insert(img_path, cache_path)
        self.assertTrue(exists(cache_path))

    def test__insert_bad_path_raises(self):
        img_path = "/qux/qux/qux"

        cache_name = "da39a3ee5e6b4b0d3255bfef95601890afd80709.img"
        cache_path = join(get_cache_path(), cache_name)

        with self.assertRaises(IOError) as cm:
            boom.cache._insert(img_path, cache_path)

    def test__remove_boot(self):
        img_name = self._make_null_testimg(restored=True)
        img_path = join(get_boot_path(), img_name)

        self.assertTrue(exists(img_path))
        boom.cache._remove_boot(img_path)
        self.assertFalse(exists(img_path))

    def test__remove_boot_bad_path_raises(self):
        with self.assertRaises(ValueError) as cm:
            boom.cache._remove_boot("nosuch.img")

    def test__remove_boot_not_restored_raises(self):
        img_name = self._make_null_testimg(restored=False)
        img_path = join(get_boot_path(), img_name)

        with self.assertRaises(ValueError) as cm:
            boom.cache._remove_boot(img_path)

    def test__remove(self):
        img_name = self._make_null_testimg()
        img_path = join(get_boot_path(), img_name)

        cache_name = "da39a3ee5e6b4b0d3255bfef95601890afd80709.img"
        cache_path = join(get_cache_path(), cache_name)

        boom.cache._insert(img_path, cache_path)
        self.assertTrue(exists(cache_path))

        boom.cache._remove(cache_path)
        self.assertFalse(exists(cache_path))

    def test__remove_nonex_path_raises(self):
        cache_name = "da39a3ee5e6b4b0d3255bfef95601890afd80709.img"
        cache_path = join(get_cache_path(), cache_name)

        with self.assertRaises(OSError) as cm:
            boom.cache._remove(cache_path)

    def test__remove_bad_path_raises(self):
        cache_path = "/da39a3ee5e6b4b0d3255bfef95601890afd80709.img"

        with self.assertRaises(ValueError) as cm:
            boom.cache._remove(cache_path)

    def test_cache_path(self):
        img_name = self._make_null_testimg(restored=False)
        img_path = join("/", img_name)

        cache_path(img_path)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 1)

    def test_cache_path_dupe(self):
        img_name = self._make_null_testimg(restored=False)
        img_path = join("/", img_name)

        ce1 = cache_path(img_path)
        ce2 = cache_path(img_path)

        self.assertEqual(repr(ce1), repr(ce2))

    def test_cache_path_nonex_path_raises(self):
        img_name = "nonexistent"
        img_path = join("/", img_name)

        with self.assertRaises(OSError) as cm:
            cache_path(img_path)

    def test_cache_path_nonreg_path_raises(self):
        img_path = "/"

        with self.assertRaises(ValueError) as cm:
            cache_path(img_path)

    def test_backup_path(self):
        img_name = self._make_null_testimg(restored=False)
        img_path = join("/", img_name)
        backup_img = img_path + ".boom0"

        backup_path(img_path, backup_img)

        # Assert backup is in cache
        ces = find_cache_paths(Selection(path=backup_img))
        self.assertEqual(len(ces), 1)

        # Assert original is not in cache
        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 0)

    def test_uncache_path(self):
        img_name = self._make_null_testimg(restored=False)
        img_path = join("/", img_name)

        cache_path(img_path)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 1)

        uncache_path(img_path)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 0)


    def test_uncache_path_not_cached(self):
        img_name = self._make_null_testimg(restored=False)
        img_path = join("/", img_name)

        with self.assertRaises(ValueError) as cm:
            uncache_path(img_path)

    def test_uncache_in_use(self):
        img_path = "/vmlinuz-4.16.11-100.fc26.x86_64"
        uncache_path(img_path)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 1)

    def test_uncache_in_use_force(self):
        img_path = "/vmlinuz-4.16.11-100.fc26.x86_64"
        uncache_path(img_path, force=True)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 0)

    def test_uncache_restored(self):
        img_path = "/initramfs-3.10.1-1.el7.img"
        uncache_path(img_path)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 1)

    def test_uncache_restored_force(self):
        img_path = "/initramfs-3.10.1-1.el7.img"
        uncache_path(img_path, force=True)

        ces = find_cache_paths(Selection(path=img_path))
        self.assertEqual(len(ces), 0)

    def test_clean_cache(self):
        clean_cache()

    def test_clean_cache_unrefed(self):
        cache_path("/initramfs-2.6.0.img")
        clean_cache()

    def test_cache_state_missing_and_restore(self):
        img_path="/initramfs-2.6.0.img"
        ce = cache_path(img_path)
        unlink(join(get_boot_path(), img_path[1:]))
        self.assertEqual(ce.state, CACHE_MISSING)
        ce.restore()
        self.assertEqual(ce.state, CACHE_RESTORED)

    def test_cache_state_missing_and_restore_with_dest(self):
        img_path="/initramfs-2.6.0.img"
        ce = cache_path(img_path)
        unlink(join(get_boot_path(), img_path[1:]))
        self.assertEqual(ce.state, CACHE_MISSING)
        ce.restore(dest=img_path + ".boom0")
        self.assertEqual(ce.state, CACHE_RESTORED)

    def test_cache_restore_non_missing_raises(self):
        # A path that is not RESTORED|MISSING
        img_path = "/vmlinuz-4.16.11-100.fc26.x86_64"
        ce = find_cache_paths(Selection(path=img_path))[0]
        with self.assertRaises(ValueError) as cm:
            ce.restore()

    def test_cache_purge_restored(self):
        img_path="/initramfs-2.6.0.img"
        ce = cache_path(img_path)
        unlink(join(get_boot_path(), img_path[1:]))
        self.assertEqual(ce.state, CACHE_MISSING)
        ce.restore()
        self.assertEqual(ce.state, CACHE_RESTORED)
        ce.purge()
        self.assertEqual(ce.state, CACHE_MISSING)

    def test_cache_purge_not_restored(self):
        # A path that is not RESTORED|MISSING
        img_path = "/vmlinuz-4.16.11-100.fc26.x86_64"
        ce = find_cache_paths(Selection(path=img_path))[0]
        self.assertEqual(ce.state, CACHE_CACHED)
        with self.assertRaises(ValueError) as cm:
            ce.purge()

    def test_find_cache_paths(self):
        ces = find_cache_paths()
        self.assertTrue(ces)

    def test_find_cache_images(self):
        ces = find_cache_images()
        self.assertTrue(ces)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# boom/__init__.py - Boom package initialisation
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
"""This module provides classes and functions for creating, displaying,
and manipulating boot loader entries complying with the Boot Loader
Specification.

The ``boom`` package contains global definitions, functions to configure
the Boom environment, logging infrastructure for the package and a
``Selection`` class used to select one or more ``OsProfile``,
``HostProfile``, ``BootEntry``, or ``BootParams`` object according to
specified selection criteria.

Individual sub-modules provide interfaces to the various components of
Boom: operating system and host profiles, boot loader entries and boot
parameters, the boom CLI and procedural API and a simple reporting
module to produce tabular reports on Boom objects.

See the sub-module documentation for specific information on the
classes and interfaces provided, and the ``boom`` tool help output and
manual page for information on using the command line interface.
"""
from __future__ import print_function

from ._boom import *
from ._boom import __all__

__version__ = "1.4"

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
#
# boom_tests.py - Boom module tests.
#
# This file is part of the boom project.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions
# of the GNU General Public License v.2.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import unittest
import logging
import boom
from sys import stdout
from os.path import abspath

from tests import *

log = logging.getLogger()
log.level = logging.DEBUG
log.addHandler(logging.FileHandler("test.log"))

BOOT_ROOT_TEST = abspath("./tests")
# Override default BOOT_ROOT.
boom.set_boot_path(BOOT_ROOT_TEST)


class BoomTests(unittest.TestCase):
    # Module tests
    def test_import(self):
        import boom

    # Helper routine tests

    def test_parse_name_value_default(self):
        # Test each allowed quoting style
        nvp = "n=v"
        (name, value) = boom.parse_name_value(nvp)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")
        nvp = "n='v'"
        (name, value) = boom.parse_name_value(nvp)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")
        nvp = 'n="v"'
        (name, value) = boom.parse_name_value(nvp)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")
        nvp = 'n = "v"'
        (name, value) = boom.parse_name_value(nvp)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")

        # Assert that a comment following a value is permitted, with or
        # without intervening whitespace.
        nvp = 'n=v # Qux.'
        (name, value) = boom.parse_name_value(nvp)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v ")
        nvp = 'n=v#Qux.'
        (name, value) = boom.parse_name_value(nvp)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")

        # Assert that a malformed nvp raises ValueError
        with self.assertRaises(ValueError) as cm:
            nvp = "n v"
            (name, value) = boom.parse_name_value(nvp)
        with self.assertRaises(ValueError) as cm:
            nvp = "n==v"
            (name, value) = boom.parse_name_value(nvp)
        with self.assertRaises(ValueError) as cm:
            nvp = "n+=v"
            (name, value) = boom.parse_name_value(nvp)

        # Test that values with embedded assignment are accepted
        (name, value) = boom.parse_name_value('n=v=v1')
        self.assertEqual(value, "v=v1")

    def test_parse_name_value_whitespace(self):
        # Test each allowed quoting style
        nvp = "n v"
        (name, value) = boom.parse_name_value(nvp, separator=None)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")
        nvp = "n 'v'"
        (name, value) = boom.parse_name_value(nvp, separator=None)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")
        nvp = 'n "v"'
        (name, value) = boom.parse_name_value(nvp, separator=None)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")
        nvp = 'n   "v"'
        (name, value) = boom.parse_name_value(nvp, separator=None)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")

        # Assert that a comment following a value is permitted, with or
        # without intervening whitespace. Trailing whitespace is
        # included in the parsed value.
        nvp = 'n v # Qux.'
        (name, value) = boom.parse_name_value(nvp, separator=None)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v ")
        nvp = 'n v#Qux.'
        (name, value) = boom.parse_name_value(nvp, separator=None)
        self.assertEqual(name, "n")
        self.assertEqual(value, "v")

        # Assert that a malformed nvp raises ValueError
        with self.assertRaises(ValueError) as cm:
            nvp = "n=v"
            (name, value) = boom.parse_name_value(nvp, separator=None)
        with self.assertRaises(ValueError) as cm:
            nvp = "n==v"
            (name, value) = boom.parse_name_value(nvp, separator=None)
        with self.assertRaises(ValueError) as cm:
            nvp = "n+=v"
            (name, value) = boom.parse_name_value(nvp, separator=None)

        # Test that values with embedded assignment are accepted
        (name, value) = boom.parse_name_value('n v=v1', separator=None)
        self.assertEqual(value, "v=v1")

    def test_blank_or_comment(self):
        self.assertTrue(boom.blank_or_comment(""))
        self.assertTrue(boom.blank_or_comment("# this is a comment"))
        self.assertFalse(boom.blank_or_comment("THIS_IS_NOT=foo"))

    def test_set_debug_mask(self):
        boom.set_debug_mask(boom.BOOM_DEBUG_ALL)

    def test_set_debug_mask_bad_mask(self):
        with self.assertRaises(ValueError) as cm:
            boom.set_debug_mask(boom.BOOM_DEBUG_ALL + 1)

    def test_BoomLogger(self):
        bl = boom.BoomLogger("boom", 0)
        bl.debug("debug")

    def test_BoomLogger_set_debug_mask(self):
        bl = boom.BoomLogger("boom", 0)
        bl.set_debug_mask(boom.BOOM_DEBUG_ALL)

    def test_BoomLogger_set_debug_mask_bad_mask(self):
        bl = boom.BoomLogger("boom", 0)
        with self.assertRaises(ValueError) as cm:
            bl.set_debug_mask(boom.BOOM_DEBUG_ALL + 1)

    def test_BoomLogger_debug_masked(self):
        bl = boom.BoomLogger("boom", 0)
        boom.set_debug_mask(boom.BOOM_DEBUG_ALL)
        bl.set_debug_mask(boom.BOOM_DEBUG_ENTRY)
        bl.debug_masked("qux")

    def test_BoomConfig__str__(self):
        bc = boom.BoomConfig(boot_path="/boot", legacy_enable=False)
        xstr = ('[defaults]\nboot_path = /boot\nboom_path = /boot/boom\n\n'
                '[legacy]\nenable = False\nformat = grub1\nsync = True\n\n'
                '[cache]\nenable = True\nauto_clean = True\n'
                'cache_path = /boot/boom/cache\n')
        self.assertEqual(str(bc), xstr)

    def test_BoomConfig__repr__(self):
        bc = boom.BoomConfig(boot_path="/boot", legacy_enable=False)
        xrepr = ('BoomConfig(boot_path="/boot", boom_path="/boot/boom", '
                 'enable_legacy=False, legacy_format="grub1", '
                 'legacy_sync=True, cache_enable=True, auto_clean=True, '
                 'cache_path="/boot/boom/cache")')
        self.assertEqual(repr(bc), xrepr)

    def test_set_boom_config(self):
        bc = boom.BoomConfig(boot_path="/boot", legacy_enable=False)
        boom.set_boom_config(bc)

    def test_set_boom_config_bad_config(self):
        class Qux(object):
            pass

        with self.assertRaises(TypeError) as cm:
            boom.set_boom_config(None)

        with self.assertRaises(TypeError) as cm:
            boom.set_boom_config(Qux())

    def test_parse_btrfs_subvol(self):
        self.assertEqual((None, "23"), boom.parse_btrfs_subvol("23"))
        self.assertEqual(("/svol", None), boom.parse_btrfs_subvol("/svol"))
        self.assertEqual((None, None), boom.parse_btrfs_subvol(None))

    def test_Selection_from_cmd_args_subvol_id(self):
        cmd_args = MockArgs()
        s = boom.Selection.from_cmd_args(cmd_args)
        self.assertEqual(s.btrfs_subvol_id, "23")
        self.assertEqual(s.boot_id, None)

    def test_Selection_from_cmd_args_subvol(self):
        cmd_args = MockArgs()
        cmd_args.btrfs_subvolume = "/svol"
        s = boom.Selection.from_cmd_args(cmd_args)
        self.assertEqual(s.btrfs_subvol_path, "/svol")

    def test_Selection_from_cmd_args_root_lv(self):
        cmd_args = MockArgs()
        cmd_args.root_lv = "vg00/lvol0"
        s = boom.Selection.from_cmd_args(cmd_args)
        self.assertEqual(s.lvm_root_lv, "vg00/lvol0")

    def test_Selection_from_cmd_args_no_btrfs(self):
        cmd_args = MockArgs()
        cmd_args.btrfs_subvolume = ""
        cmd_args.root_lv = "vg00/lvol0"
        s = boom.Selection.from_cmd_args(cmd_args)
        self.assertEqual(s.lvm_root_lv, "vg00/lvol0")

    def test_Selection_invalid_selection(self):
        # A boot_id is invalid for an OsProfile select
        s = boom.Selection(boot_id="12345678")
        with self.assertRaises(ValueError) as cm:
            s.check_valid_selection(profile=True)

    def test_Selection_is_null(self):
        s = boom.Selection()
        self.assertTrue(s.is_null())

    def test_Selection_is_non_null(self):
        s = boom.Selection(boot_id="1")
        self.assertFalse(s.is_null())


class BoomPathTests(unittest.TestCase):
    def setUp(self):
        # Set up required test paths
        pass

    def tearDown(self):
        # Clean up test sandbox
        pass

    def test_set_boot_path(self):
        boom.set_boot_path(BOOT_ROOT_TEST)

    def test_set_boot_path_bad_path(self):
        with self.assertRaises(ValueError) as cm:
            boom.set_boot_path("/the/wrong/path")

    def test_set_boom_path(self):
        boom.set_boom_path(BOOT_ROOT_TEST + "/boom")

    def test_set_boom_path_bad_path(self):
        with self.assertRaises(ValueError) as cm:
            boom.set_boom_path("/the/wrong/path")

    def test_set_boom_path_non_abs(self):
        boom.set_boot_path(BOOT_ROOT_TEST)
        boom.set_boom_path("boom/")

    def test_set_boom_path_non_abs_bad(self):
        boom.set_boot_path(BOOT_ROOT_TEST + "/boom")
        with self.assertRaises(ValueError) as cm:
            boom.set_boom_path("absolutely/the/wrong/path")

    def test_set_boot_path_non_abs(self):
        with self.assertRaises(ValueError) as cm:
            boom.set_boot_path("absolutely/the/wrong/path")

    def test_set_boom_path_no_profiles(self):
        boom.set_boot_path(BOOT_ROOT_TEST)
        with self.assertRaises(ValueError) as cm:
            boom.set_boom_path("loader")

    def test_set_get_cache_path(self):
        cache_path = BOOT_ROOT_TEST + "/boom/cache"
        boom.set_cache_path(cache_path)
        self.assertEqual(boom.get_cache_path(), cache_path)
# vim: set et ts=4 sw=4 :
